@@ -25,7 +25,7 @@
 #include "bitmap.h"
 
 
-char pbm_getrawbyte( FILE * f )
+char pbm_getrawbyte( FILE * f ) throw( Bitmap::Error )
   {
   int ch = getc( f );
 
@@ -35,7 +35,7 @@ char pbm_getrawbyte( FILE * f )
   return static_cast< char > (ch);
   }
 
-char pbm_getc( FILE * f )
+char pbm_getc( FILE * f ) throw()
   {
   char ch;
   bool comment = false;
@@ -49,7 +49,7 @@ char pbm_getc( FILE * f )
   return ch;
   }
 
-int pbm_getint( FILE * f )
+int pbm_getint( FILE * f ) throw( Bitmap::Error )
   {
   char ch;
   int i = 0;
@@ -62,7 +62,7 @@ int pbm_getint( FILE * f )
   return i;
   }
 
-bool pbm_getbit( FILE * f )
+bool pbm_getbit( FILE * f ) throw( Bitmap::Error )
   {
   char ch;
 
@@ -74,7 +74,7 @@ bool pbm_getbit( FILE * f )
   }
 
 
-Bitmap::Bitmap( FILE * f )
+Bitmap::Bitmap( FILE * f ) throw( Error )
   {
   char filetype = 0;
 
@@ -113,10 +113,10 @@ Bitmap::Bitmap( FILE * f )
   }
 
 
-Bitmap::Bitmap( const Bitmap & source, const Rectangle & r )
+Bitmap::Bitmap( const Bitmap & source, const Rectangle & r ) throw()
   {
   if( r.right() >= source._width || r.bottom() >= source._height )
-    throw Internal_error( "bad parameter building a Bitmap from another one.\n" );
+    internal_error( "bad parameter building a Bitmap from another one" );
 
   _width = r.right() - r.left() + 1;
   _height = r.bottom() - r.top() + 1;
@@ -132,11 +132,11 @@ Bitmap::Bitmap( const Bitmap & source, const Rectangle & r )
   }
 
 
-Bitmap::Bitmap( const Bitmap & source, int th, const int scale )
+Bitmap::Bitmap( const Bitmap & source, int th, int scale ) throw()
   {
   if( scale < 2 || scale > source._width || scale > source._height ||
       th >= scale * scale )
-    throw Internal_error( "bad parameter building a reduced Bitmap.\n" );
+    internal_error( "bad parameter building a reduced Bitmap" );
 
   if( th < 0 )
     {
@@ -173,7 +173,7 @@ Bitmap::Bitmap( const Bitmap & source, int th, const int scale )
   }
 
 
-void floodfill4( const Bitmap & source, Bitmap & dest, int row, int col )
+void floodfill4( const Bitmap & source, Bitmap & dest, int row, int col ) throw()
   {
   std::stack< int, std::vector< int > > st;
 
@@ -196,7 +196,7 @@ void floodfill4( const Bitmap & source, Bitmap & dest, int row, int col )
   }
 
 
-Bitmap::Bitmap( const Bitmap & source, const type t )
+Bitmap::Bitmap( const Bitmap & source, type t ) throw( Error )
   {
   _width = source._width; _height = source._height;
 
@@ -227,12 +227,67 @@ Bitmap::Bitmap( const Bitmap & source, const type t )
     if( row == _height ) throw Error( "image is not clear enough.\n" );
     floodfill4( source, *this, row, _width/2 );
     }
-  else
-    throw Internal_error( "type not implemented building a 'special' Bitmap.\n" );
+  else internal_error( "type not implemented building a 'special' Bitmap" );
   }
 
 
-int Bitmap::horizontalify( const bool verbose )
+int Bitmap::find_columns( bool verbose ) throw()
+  {
+  if( _rectangle_list.size() != 0 )
+    internal_error( "find_columns called twice" );
+
+  const int colmin = ( _width / 10 ) + 100;
+  const int gapmin = ( _width / 100 ) + 10;
+  std::vector< int > h_outline;
+  h_outline.resize( _width );
+
+  int total_dots = 0;
+  for( int col = 0; col < _width; ++col )
+    {
+    h_outline[col] = 0;
+    for( int row = 0; row < _height; ++row )
+      if( data[row][col] ) ++h_outline[col];
+    total_dots += h_outline[col];
+    }
+  int threshold_col = ( total_dots / ( _width * 4 ) ) + 6;
+  int threshold_gap = ( total_dots / ( _width * 20 ) ) + 1;
+
+  int left, right, col_width, gap_width;
+  for( left = 0; left < _width; ++left )	// cut left border
+    if( h_outline[left] >= threshold_gap ) break;
+  right = left + 1;
+  left = ( left >= gapmin ) ? left / 2 : 0;
+
+  while( true )
+    {
+    for( col_width = 0; right < _width; ++right )  // look for text
+      {
+      if( h_outline[right] > threshold_col ) ++col_width;
+      else if( col_width < colmin ) col_width = 0;
+      else break;
+      }
+    for( gap_width = 0; right < _width; ++right )  // look for gap
+      {
+      if( h_outline[right] < threshold_gap ) ++gap_width;
+      else if( gap_width < gapmin ) gap_width = 0;
+      else break;
+      }
+    if( right >= _width )
+      {
+      Rectangle r( left, 0, _width - 1, _height - 1 );
+      _rectangle_list.push_back( r ); break;
+      }
+    Rectangle r( left, 0, right - ( gap_width / 2 ), _height - 1 );
+    _rectangle_list.push_back( r );
+    left = right - ( gap_width / 2 );
+    }
+  if( verbose )
+    fprintf( stderr, "number of text blocks %d\n", _rectangle_list.size() );
+  return _rectangle_list.size();
+  }
+
+
+int Bitmap::horizontalify( bool verbose ) throw()
   {
   const int slope_max = _width / 25;
   int slope_best = 0;			// not a true slope
@@ -344,69 +399,7 @@ if( verbose )
   }
 
 
-int Bitmap::find_columns( const bool verbose )
-  {
-  const int colmin = (_width / 10) + 60;
-  const int gapmin = (_width / 100) + 6;
-  std::vector< int > h_outline;
-  int total_dots = 0, threshold_col, threshold_gap;
-  int row, col;
-
-  if( _rectangle_list.size() != 0 )
-    throw Internal_error( "find_columns called twice.\n" );
-
-  h_outline.resize( _width );
-
-  for( col = 0; col < _width; ++col )
-    {
-    h_outline[col] = 0;
-    for( row = 0; row < _height; ++row )
-      if( data[row][col] ) ++h_outline[col];
-    total_dots += h_outline[col];
-    }
-
-  threshold_col = (total_dots / (_width * 4)) + 6;
-  threshold_gap = (total_dots / (_width * 20)) + 1;
-
-  int left = 0, right, run_length;
-  bool left_border = true;
-  for( col = 0;; )
-    {
-    if( !left_border )			// look for text column
-      {
-      for( run_length = 0; col < _width-1; ++col )
-        {
-        if( h_outline[col] > threshold_col ) ++run_length;
-        else { if( run_length < colmin ) run_length = 0; else break; }
-        }
-      }
-
-    // look for gap
-    for( run_length = 0; col < _width-1; ++col )
-      {
-      if( h_outline[col] < threshold_gap ) ++run_length;
-      else { if( run_length < gapmin ) run_length = 0; else break; }
-      }
-
-    if( col >= _width-1 )
-      {
-      _rectangle_list.push_back( Rectangle( left, 0, _width-1, _height-1 ) );
-      break;
-      }
-
-    right = col - (run_length / 2);
-    if( left_border ) left_border = false;
-    else _rectangle_list.push_back( Rectangle( left, 0, right, _height-1 ) );
-    left = right;
-    }
-
-  if( verbose )
-    fprintf( stderr, "number of text blocks %d\n", _rectangle_list.size() );
-  return _rectangle_list.size();
-  }
-
-
-void Bitmap::save( FILE * f, char filetype )
+void Bitmap::save( FILE * f, char filetype ) const throw()
   {
   if( filetype != '1' && filetype != '4' ) filetype = '4';
   fprintf( f, "P%c\n%d %d\n", filetype, _width, _height );
