@@ -19,23 +19,24 @@ public:
 
   enum type { vertical_histogram, horizontal_histogram, connected_ground };
 
-  Bitmap( FILE * );			// Creates a Bitmap from a pbm file
+  Bitmap( FILE * ) throw( Error );	// Creates a Bitmap from a pbm file
   // Creates a Bitmap from another Bitmap
-  Bitmap( const Bitmap & source, const Rectangle & r );
+  Bitmap( const Bitmap & source, const Rectangle & r ) throw();
   // Creates a reduced bitmap
-  Bitmap( const Bitmap & source, int th, const int scale );
-  Bitmap( const Bitmap &, const type );	// Creates a "special" type of Bitmap
+  Bitmap( const Bitmap & source, int th, int scale ) throw();
+  // Creates a "special" type of Bitmap
+  Bitmap( const Bitmap &, type ) throw( Error );
 
   int width()  const throw() { return _width;  }
   int height() const throw() { return _height; }
-  bool get_bit( const int row, const int col ) const throw()
-    { return data[row][col]; }
+  bool get_bit( int row, int col ) const throw() { return data[row][col]; }
+
   const std::vector< Rectangle > & rectangle_list() const throw()
     { return _rectangle_list; }
-  int find_columns( const bool verbose = false );
-  int horizontalify( const bool verbose = false );
-  void save( FILE *, char filetype = '4' );
+  int find_columns( bool verbose = false ) throw();
+  int horizontalify( bool verbose = false ) throw();
+  void save( FILE *, char filetype = '4' ) const throw();
 
-  friend void floodfill4( const Bitmap & source, Bitmap & dest,
-			const int row, const int col );
+  friend void floodfill4( const Bitmap & source, Bitmap & dest, int row,
+                                                 int col ) throw();
   };
