@@ -26,13 +26,13 @@
 #include "blockmap.h"
 
 
-void Blockmap::create_block( const Rectangle & r, const int id ) throw()
+void Blockmap::create_block( const Rectangle & r, int id ) throw()
   {
   Block b( r, *this, id );
   _block_list.push_back( b );
   }
 
-void Blockmap::delete_block( const int id ) throw( Internal_error )
+void Blockmap::delete_block( int id ) throw()
   {
   std::list< Block >::reverse_iterator p = _block_list.rbegin();
   for( ; p != _block_list.rend(); ++p )
@@ -44,10 +44,10 @@ void Blockmap::delete_block( const int id ) throw( Internal_error )
       _block_list.erase( --(p.base()) ); return;
       }
 
-  throw Internal_error( "lost block (delete).\n" );
+  internal_error( "delete_block, lost block" );
   }
 
-void Blockmap::join_blocks( int id1, int id2 ) throw( Internal_error )
+void Blockmap::join_blocks( int id1, int id2 ) throw()
   {
   if( id1 > id2 )
     { int temp = id1; id1 = id2; id2 = temp; }
@@ -57,7 +57,7 @@ void Blockmap::join_blocks( int id1, int id2 ) throw( Internal_error )
   while( p1 != _block_list.rend() && id1 != p1->id() ) ++p1;
   while( p2 != _block_list.rend() && id2 != p2->id() ) ++p2;
   if( p1 == _block_list.rend() || p2 == _block_list.rend() )
-    throw Internal_error( "lost block (join).\n" );
+    internal_error( "join_blocks, lost block" );
 
   for( int row = p2->top(); row <= p2->bottom(); ++row )
     for( int col = p2->left(); col <= p2->right(); ++col )
@@ -67,18 +67,17 @@ void Blockmap::join_blocks( int id1, int id2 ) throw( Internal_error )
   _block_list.erase( --(p2.base()) );
   }
 
-void Blockmap::add_point_to_block( const int row, const int col, const int id )
-                                                        throw( Internal_error )
+void Blockmap::add_point_to_block( int row, int col, int id ) throw()
   {
   std::list< Block >::reverse_iterator p = _block_list.rbegin();
   for( ; p != _block_list.rend(); ++p )
     if( id == p->id() ) { p->add_point( row, col ); return; }
 
-  throw Internal_error( "lost block (add_point).\n" );
+  internal_error( "add_point_to_block, lost block" );
   }
 
 
-int Blockmap::left_neighbor( const Rectangle & r , const int distance )
+int Blockmap::left_neighbor( const Rectangle & r , int distance )
 							const throw()
   {
   int limit = distance >= r.left() ? 0 : r.left() - distance;
@@ -92,7 +91,7 @@ int Blockmap::left_neighbor( const Rectangle & r , const int distance )
   return 0;
   }
 
-int Blockmap::top_neighbor( const Rectangle & r , const int distance )
+int Blockmap::top_neighbor( const Rectangle & r , int distance )
 								const throw()
   {
   int limit = distance >= r.top() ? 0 : r.top() - distance;
@@ -106,7 +105,7 @@ int Blockmap::top_neighbor( const Rectangle & r , const int distance )
   return 0;
   }
 
-int Blockmap::right_neighbor( const Rectangle & r , const int distance )
+int Blockmap::right_neighbor( const Rectangle & r , int distance )
 								const throw()
   {
   int limit = r.right() + distance + 1;
@@ -119,7 +118,7 @@ int Blockmap::right_neighbor( const Rectangle & r , const int distance )
   return 0;
   }
 
-int Blockmap::bottom_neighbor( const Rectangle & r , const int distance )
+int Blockmap::bottom_neighbor( const Rectangle & r , int distance )
 								const throw()
   {
   int limit = r.bottom() + distance + 1;
@@ -216,19 +215,25 @@ int generate_black_id() { static int next = 0; return ++next; }
 
 int generate_white_id() { static int next = 0; return --next; }
 
-Blockmap::Blockmap( const Bitmap & page_image, const int debug_level )
-						throw( Internal_error )
+Blockmap::Blockmap( const Bitmap & page_image, int column, int debug_level )
+                                                                     throw()
   {
-  _height = page_image.height();
-  _width = page_image.width();
+  if( column < 0 ||
+      column >= static_cast<int>(page_image.rectangle_list().size()) )
+    internal_error( "Blockmap, column out of range" );
 
+  int row0 = page_image.rectangle_list()[column].top();
+  int col0 = page_image.rectangle_list()[column].left();
+  _height =  page_image.rectangle_list()[column].height();
+  _width =   page_image.rectangle_list()[column].width();
   data.resize( _height );
+
   for( int row = 0; row < _height; ++row )
     {
     data[row].resize( _width );
     for( int col = 0; col < _width; ++col )
       {
-      if( page_image.get_bit( row, col ) == true )	// point is black
+      if( page_image.get_bit( row0 + row, col0 + col ) )	// black point
         {
         if( row > 0 && data[row-1][col] > 0 )
           {
@@ -248,7 +253,7 @@ Blockmap::Blockmap( const Bitmap & page_image, const int debug_level )
           create_block( Rectangle( col, row, col, row ), data[row][col] );
           }
         }
-      else						// point is white
+      else						// white point
         {
         if( row == 0 || col == 0 ) data[row][col] = 0;
         else if( data[row-1][col] == 0 || data[row][col-1] == 0 )
@@ -281,3 +286,15 @@ Blockmap::Blockmap( const Bitmap & page_image, const int debug_level )
   if( debug_level <= 99 ) hierarchize_blocks( _block_list );
   if( debug_level <= 97 ) order_blocks( _block_list );
   }
+
+
+void Blockmap::print( FILE * outfile, int debug_level ) const throw()
+  {
+  fprintf( outfile, "%d blocks\n", _block_list.size() );
+  fprintf( outfile, "%d %d\n\n", _width, _height );
+
+  int sp = (debug_level & 1) ? 0 : -1;
+  std::list< Block >::const_iterator p = _block_list.begin();
+  for( ; p != _block_list.end(); ++p )
+    p->print( outfile, sp );
+  }
