@@ -16,11 +16,19 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
+#include <cstdio>
 #include <cstdlib>
 #include "common.h"
 
 
-bool similar( const int a, const int b, const int percent_dif )
+void internal_error( const char * s ) throw()
+  {
+  fprintf( stderr, "%s internal error: %s.\n", PROGRAM_NAME, s );
+  exit( 3 );
+  }
+
+
+bool similar( int a, int b, int percent_dif ) throw()
   {
   int difference = abs( a - b );
   if( percent_dif > 0 && difference <= 1 ) return true;
