@@ -27,16 +27,17 @@ public:
   const std::vector< int > & hscan() const throw();
   const std::vector< int > & vscan() const throw();
 
-  bool escape_left  ( int row, int col ) const throw();
-  bool escape_top   ( int row, int col ) const throw();
-  bool escape_right ( int row, int col ) const throw();
-  bool escape_bottom( int row, int col ) const throw();
-  int seek_left  ( int row, int col ) const throw();
-  int seek_top   ( int row, int col ) const throw();
-  int seek_right ( int row, int col ) const throw();
-  int seek_bottom( int row, int col ) const throw();
+  unsigned char test_235Esz() const throw();
+  unsigned char test_49egpq() const throw();
+  unsigned char test_6abd() const throw();
+  unsigned char test_CEFIJLlT() const throw();
+  unsigned char test_cG() const throw();
+  unsigned char test_frt() const throw();
+  unsigned char test_HKMNUuvwYy() const throw();
+  unsigned char test_hnw() const throw();
 
-  char test_235sz() const throw();
-  char test_hn() const throw();
-  char test_line() const throw();
+  bool test_comma() const throw();
+  unsigned char test_line() const throw();
+  unsigned char test_solid() const throw();
+  unsigned char test_misc() const throw();
   };
