@@ -22,99 +22,92 @@
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
-#include "blockmap.h"
 #include "profile.h"
 
 
 Profile::Profile( const Block & b, type t ) throw()
   : _block( &b ), _blockmap( b.blockmap() ), _type(t), valid(false),
-    valid_mean(false), valid_range(false), valid_isconcave(false),
+    valid_max(false), valid_min(false), valid_mean(false),
+    valid_range(false), valid_isconcave(false),
     valid_isconvex(false), valid_isflat(false), valid_isflats(false),
-    valid_ispit(false), valid_isvpit(false), valid_istip(false),
-    valid_isctip(false) {}
+    valid_ispit(false), valid_isupit(false), valid_isvpit(false),
+    valid_istip(false) {}
 
 
 void Profile::initialize() throw()
   {
   const Block & b = *_block;
+
   valid = true;
   switch( _type )
     {
     case left :
-      data.resize( b.height() ); limit = b.width();
+      data.resize( b.height() ); _limit = b.width();
       for( int i = 0; i < samples(); ++i )
         {
-        int j = 0, row = b.top() + i;
-        while( j < b.width() && b.blockmap()->id( row, b.left() + j ) != b.id() )
-          ++j;
-        data[i] = j;
+        int j = b.left(), row = b.top() + i;
+        while( j <= b.right() && b.id( row, j ) != b.id() ) ++j;
+        data[i] = j - b.left();
         } break;
     case top :
-      data.resize( b.width() ); limit = b.height();
+      data.resize( b.width() ); _limit = b.height();
       for( int i = 0; i < samples(); ++i )
         {
-        int j = 0, col = b.left() + i;
-        while( j < b.height() && b.blockmap()->id( b.top() + j, col ) != b.id() )
-          ++j;
-        data[i] = j;
+        int j = b.top(), col = b.left() + i;
+        while( j <= b.bottom() && b.id( j, col ) != b.id() ) ++j;
+        data[i] = j - b.top();
         } break;
     case right :
-      data.resize( b.height() ); limit = b.width();
+      data.resize( b.height() ); _limit = b.width();
       for( int i = 0; i < samples(); ++i )
         {
-        int j = 0, row = b.top() + i;
-        while( j < b.width() && b.blockmap()->id( row, b.right() - j ) != b.id() )
-          ++j;
-        data[i] = j;
+        int j = b.right(), row = b.top() + i;
+        while( j >= b.left() && b.id( row, j ) != b.id() ) --j;
+        data[i] = b.right() - j;
         } break;
     case bottom :
-      data.resize( b.width() ); limit = b.height();
+      data.resize( b.width() ); _limit = b.height();
       for( int i = 0; i < samples(); ++i )
         {
-        int j = 0, col = b.left() + i;
-        while( j < b.height() && b.blockmap()->id( b.bottom() - j, col ) != b.id() )
-          ++j;
-        data[i] = j;
+        int j = b.bottom(), col = b.left() + i;
+        while( j >= b.top() && b.id( j, col ) != b.id() ) --j;
+        data[i] = b.bottom() - j;
         } break;
     case height :
-      data.resize( b.width() ); limit = b.height();
+      data.resize( b.width() ); _limit = b.height();
       for( int i = 0; i < samples(); ++i )
         {
-        int j1 = 0, j2 = 0, col = b.left() + i;
-        while( j1 < b.height() && b.blockmap()->id( b.top() + j1, col ) != b.id() )
-          ++j1;
-        while( j2 < b.height() && b.blockmap()->id( b.bottom() - j2, col ) != b.id() )
-          ++j2;
-        data[i] = (j1 + j2 < b.height()) ? b.height() - j1 - j2 : 0;
+        int u = b.top(), d = b.bottom(), col = b.left() + i;
+        while( u <= d && b.id( u, col ) != b.id() ) ++u;
+        while( u <= d && b.id( d, col ) != b.id() ) --d;
+        data[i] = ( u <= d ) ? d - u + 1 : 0;
         } break;
     case width :
-      data.resize( b.height() ); limit = b.width();
+      data.resize( b.height() ); _limit = b.width();
       for( int i = 0; i < samples(); ++i )
         {
-        int j1 = 0, j2 = 0, row = b.top() + i;
-        while( j1 < b.width() && b.blockmap()->id( row, b.left() + j1 ) != b.id() )
-          ++j1;
-        while( j2 < b.width() && b.blockmap()->id( row, b.right() - j2 ) != b.id() )
-          ++j2;
-        data[i] = (j1 + j2 < b.width()) ? b.width() - j1 - j2 : 0;
+        int l = b.left(), r = b.right(), row = b.top() + i;
+        while( l <= r && b.id( row, l ) != b.id() ) ++l;
+        while( l <= r && b.id( row, r ) != b.id() ) --r;
+        data[i] = ( l <= r ) ? r - l + 1 : 0;
         } break;
     case zheight :
-      data.resize( b.width() ); limit = b.height();
+      data.resize( b.width() ); _limit = b.height();
       for( int i = 0; i < samples(); ++i )
         {
         int col = b.left() + i;
         data[i] = 0;
-        for( int j = 0; j < b.height(); ++j )
-          if( b.blockmap()->id( b.top() + j, col ) == b.id() ) ++data[i];
+        for( int j = b.top(); j <= b.bottom(); ++j )
+          if( b.id( j, col ) == b.id() ) ++data[i];
         } break;
     case zwidth :
-      data.resize( b.height() ); limit = b.width();
+      data.resize( b.height() ); _limit = b.width();
       for( int i = 0; i < samples(); ++i )
         {
         int row = b.top() + i;
         data[i] = 0;
-        for( int j = 0; j < b.width(); ++j )
-          if( b.blockmap()->id( row, b.left() + j ) == b.id() ) ++data[i];
+        for( int j = b.left(); j <= b.right(); ++j )
+          if( b.id( row, j ) == b.id() ) ++data[i];
         } break;
     }
   }
@@ -133,6 +126,48 @@ int Profile::mean() throw()
   }
 
 
+int Profile::limit() throw()
+  {
+  if( !valid ) initialize(); return _limit;
+  }
+
+
+int Profile::max() throw()
+  {
+  if( !valid_max )
+    {
+    valid_max = true; if( !valid ) initialize();
+    _max = data[0];
+    for( int i = 1; i < samples(); ++i ) if( data[i] > _max ) _max = data[i];
+    }
+  return _max;
+  }
+
+
+int Profile::min() throw()
+  {
+  if( !valid_min )
+    {
+    valid_min = true; if( !valid ) initialize();
+    _min = data[0];
+    for( int i = 1; i < samples(); ++i ) if( data[i] < _min ) _min = data[i];
+    }
+  return _min;
+  }
+
+
+int Profile::operator[]( int i ) throw()
+  {
+  if( !valid ) initialize();
+  if( i < 0 ) i = 0;
+  else if( i >= samples() ) i = samples() - 1;
+  return data[i];
+  }
+
+int Profile::pos( int p ) throw()
+  { return ( ( samples() - 1 ) * p ) / 100; }
+
+
 int Profile::range() throw()
   {
   if( !valid_range )
@@ -146,26 +181,6 @@ int Profile::range() throw()
   }
 
 
-int Profile::operator[]( int i ) throw()
-  {
-  if( !valid ) initialize();
-  if( i < 0 ) i = 0;
-  else if( i >= samples() ) i = samples() - 1;
-  return data[i];
-  }
-
-/*
-bool Profile::equal( Profile & p ) throw()
-  {
-  if( !valid ) initialize();
-  if( !p.valid ) p.initialize();
-  if( samples() != p.samples() ) return false;
-  for( int i = 0; i < samples(); ++i )
-    if( data[i] != p.data[i] ) return false;
-  return true;
-  }
-*/
-
 bool Profile::increasing( int i ) throw()
   {
   if( !valid ) initialize();
@@ -185,57 +200,40 @@ bool Profile::decreasing( int i ) throw()
   return true;
   }
 
-/*
-bool Profile::isconcave() throw()
-  {
-  if( !valid_isconcave )
-    {
-    valid_isconcave = true; _isconcave = false; if( !valid ) initialize();
-    int max = limit, imax = 0;
-
-    for( int i = 0; i < samples(); ++i )
-      if( data[i] > max ) { max = data[i]; imax = i; }
-    if( imax < samples() / 4 || imax > (samples() * 3) / 4 ) return _isconcave;
-
-    int min = limit;
-    for( int i = 1; i < samples(); ++i )
-      {
-      if( data[i] - data[i-1] < min ) min = data[i] - data[i-1];
-      else if( data[i] - data[i-1] > min + 1 ) return _isconcave;
-      }
-    _isconcave = true;
-    }
-  return _isconcave;
-  }
-*/
 
 bool Profile::isconvex() throw()
   {
   if( !valid_isconvex )
     {
     valid_isconvex = true; _isconvex = false; if( !valid ) initialize();
-    int min = limit, lmin = limit, rmax = -limit, imin = 0, l = 0, r = 0;
+    int min = _limit, min_begin = 0, min_end = 0;
+    int lmin = _limit, rmax = -_limit, l = 0, r = 0;
 
     for( int i = 1; i < samples(); ++i )
       {
       int d = data[i] - data[i-1];
       if( d < lmin ) { lmin = d; l = i; }
       if( d >= rmax ) { rmax = d; r = i; }
-      if( data[i] < min ) { min = data[i]; imin = i; }
+      if( data[i] <= min )
+        { min_end = i; if( data[i] < min ) { min = data[i]; min_begin = i; } }
       }
-    if( imin < samples() / 4 || imin > (samples() * 3) / 4 ) return _isconvex;
-    if( l >= r ) return _isconvex;
-    if( l >= samples() / 4 ) return _isconvex;
-    if( r <= (samples() * 3) / 4 ) return _isconvex;
+    if( l >= r || l >= pos( 25 ) || r <= pos( 75 ) ) return _isconvex;
+    if( lmin >= 0 || rmax <= 0 ) return _isconvex;
+    if( min_end - min_begin > samples() / 2 ) return _isconvex;
+    if( l >= min_begin || r <= min_end ) return _isconvex;
+    int imin = ( min_begin + min_end ) / 2;
+    if( imin < pos( 25 ) || imin > pos( 80 ) ) return _isconvex;
 
-    int dmax = -limit;
+    int dmax = -_limit;
     for( int i = l; i <= r; ++i )
       {
+      if( i > min_begin && i < min_end )
+        { if( data[i] <= 1 ) continue; else return _isconvex; }
       int d = data[i] - data[i-1];
       if( d > dmax ) dmax = d;
       else if( d < dmax - 2 || ( d < dmax - 1 && d != 0 ) ) return _isconvex;
       }
-    if( dmax > 0 ) _isconvex = true;
+    _isconvex = true;
     }
   return _isconvex;
   }
@@ -260,9 +258,9 @@ bool Profile::isflats() throw()
   if( !valid_isflats )
     {
     valid_isflats = true; _isflats = false; if( !valid ) initialize();
-    int begin = (samples() + 9) / 10;
-    int end = (samples() * 9) / 10;
-    if( begin >= end ) return _isflats;
+    int begin = pos( 10 ); if( begin < 3 ) begin = 3;
+    int end = pos( 90 ); if( end > samples() - 4 ) end = samples() - 4;
+    if( 2 * ( end - begin ) <= samples() ) return _isflats;
 
     int mean1 = 0;
     for( int i = begin; i < end; ++i ) mean1 += data[i];
@@ -304,6 +302,43 @@ bool Profile::ispit() throw()
   }
 
 
+bool Profile::isupit() throw()
+  {
+  if( !valid_isupit )
+    {
+    valid_isupit = true; _isupit = false; if( !valid ) initialize();
+    if( samples() < 5 ) return _isupit;
+
+    int th = ( mean() < 2 && range() > 2 ) ? 2 : mean();
+    int status = 0, ucount = 0, lcount = 0, umean =0, lmean = 0;
+    for( int i = 0; i < samples(); ++i )
+      {
+      int d = data[i];
+      switch( status )
+        {
+        case 0: if( d < th )
+                  { if( i < pos( 25 ) || i > pos( 70 ) ) return _isupit;
+                  status = 1; break; }
+                if( d > th ) { ++ucount; umean += d; }
+          break;
+        case 1: if( d > th )
+                  { if( i < pos( 30 ) || i > pos( 75 ) ) return _isupit;
+                  status = 2; break; }
+                if( d < th ) { ++lcount; lmean += d; }
+          break;
+        case 2: if( d < th ) return _isupit;
+                if( d > th ) { ++ucount; umean += d; }
+          break;
+        }
+      }
+    if( ucount > 1 ) umean /= ucount;
+    if( lcount > 1 ) lmean /= lcount;
+    _isupit = ( status == 2 && umean - lmean > range() / 2 );
+    }
+  return _isupit;
+  }
+
+
 bool Profile::isvpit() throw()
   {
   if( !valid_isvpit )
@@ -344,31 +379,27 @@ bool Profile::istip() throw()
   }
 
 
-bool Profile::isctip() throw()
+bool Profile::isctip( int cpos ) throw()
   {
-  if( !valid_isctip )
+  if( !valid ) initialize();
+  if( samples() < 5 ) return false;
+  int th = ( mean() < 2 ) ? 2 : mean();
+  int imax = -1, mid = ( ( samples() - 1 ) * cpos ) / 100;
+
+  for( int i = 0; i < samples() / 4; ++i )
     {
-    valid_isctip = true; _isctip = false; if( !valid ) initialize();
-    if( samples() < 5 ) return _isctip;
-    int th = ( mean() < 2 ) ? 2 : mean();
-    int imax = -1, mid = samples() / 2;
-
-    for( int i = 0; i < samples() / 4; ++i )
-      {
-      if( data[mid+i] > th ) { imax = mid + i; break; }
-      if( data[mid-i-1] > th ) { imax = mid - i - 1; break; }
-      }
-    if( imax < 0 ) return _isctip;
-
-    for( int i = imax + 1; i < samples(); ++i )
-      if( data[i] < th )
-        {
-        for( int j = imax - 1; j >= 0; --j )
-          if( data[j] < th ) { _isctip = true; return _isctip; }
-        break;
-        }
+    if( data[mid+i] > th ) { imax = mid + i; break; }
+    if( data[mid-i-1] > th ) { imax = mid - i - 1; break; }
     }
-  return _isctip;
+  if( imax < 0 ) return false;
+
+  for( int i = imax + 1; i < samples(); ++i )
+    if( data[i] < th )
+      {
+      for( int j = imax - 1; j >= 0; --j ) if( data[j] < th ) return true;
+      break;
+      }
+  return false;
   }
 
 
@@ -384,15 +415,28 @@ int Profile::imaximum() throw()
   }
 
 
-int Profile::iminimum() throw()
+int Profile::iminimum( int m, int th ) throw()
   {
   if( !valid ) initialize();
-  int mbegin = 0, mend, mvalue = limit + 1;
-  for( int i = 0; i < samples(); ++i )
-    if( data[i] < mvalue ) { mvalue = data[i]; mbegin = i; }
-  for( mend = mbegin + 1; mend < samples(); ++mend )
-    if( data[mend] > mvalue ) break;
-  return ( mbegin + mend - 1 ) / 2;
+  if( samples() < 1 ) return false;
+  if( th < 2 ) th = ( mean() < 2 ) ? 2 : mean();
+  int minima = 0, status = 0;
+  int begin = 0, end, value = _limit + 1;
+
+  for( end = 0; end < samples(); ++end )
+    {
+    if( status == 0 )
+      { if( data[end] < th ) { status = 1; ++minima; begin = end; } }
+    else if( data[end] > th )
+      { if( minima == m + 1 ) { --end; break; } else status = 0; }
+    else if( end == samples() - 1 ) break;
+    }
+  if( minima != m + 1 ) return 0;
+
+  for( int i = begin; i <= end; ++i )
+    if( data[i] < value ) { value = data[i]; begin = i; }
+  for( ; end >= begin; --end ) if( data[end] == value ) break;
+  return ( begin + end ) / 2;
   }
 
 
@@ -410,7 +454,7 @@ int Profile::minima( int th ) throw()
   {
   if( !valid ) initialize();
   if( samples() < 1 ) return false;
-  if( th < 0 ) th = ( mean() < 2 ) ? 2 : mean();
+  if( th < 2 ) th = ( mean() < 2 ) ? 2 : mean();
   int minima = ( data[0] < th ) ? 1 : 0;
   int status = ( minima ) ? 1 : 0;
 
@@ -424,15 +468,19 @@ int Profile::minima( int th ) throw()
   }
 
 
-bool Profile::straight( int & slope ) throw()
+bool Profile::straight( int * _dy ) throw()
   {
   if( !valid ) initialize();
   if( samples() < 5 ) return false;
-  int total_dif = data[samples()-2] - data[1];
-  int mean_dif = total_dif / ( samples() - 3 );
 
-  for( int i = 2; i <= samples() - 2; ++i )
-    if( abs( mean_dif - ( data[i] - data[i-1] ) ) > 1 ) return false;
-  slope = total_dif;
+  int xl = 1,             yl = data[1];
+  int xr = samples() - 2, yr = data[samples()-2];
+  int dx = xr - xl, dy = yr - yl;
+  for( int i = xl + 1; i < xr; ++i )
+    {
+    int y = yl; if( dx ) y += ( dy * ( i - xl ) / dx );
+    if( abs( y - data[i] ) > 1 ) return false;
+    }
+  *_dy = dy;
   return true;
   }
