@@ -24,8 +24,8 @@
 #include "common.h"
 #include "rectangle.h"
 #include "block.h"
-#include "blockmap.h"
 #include "character.h"
+#include "iso_8859_1.h"
 #include "textline.h"
 
 
@@ -35,33 +35,34 @@ void Textline::add_character( const Character & c ) throw()
   }
 
 
-void Textline::insert_character( const int i, const Character & c )
-						throw( Internal_error )
+void Textline::insert_character( int i, const Character & c ) throw()
   {
-  if( i > characters() ) throw Internal_error( "character index too big (insert)" );
+  if( i < 0 || i > characters() )
+    internal_error( "insert_character, index out of bounds" );
   data.insert( data.begin() + i, c );
   }
 
 
-void Textline::delete_character( const int i ) throw( Internal_error )
+void Textline::delete_character( int i ) throw()
   {
-  if( i > characters() ) throw Internal_error( "character index too big (delete)" );
+  if( i < 0 || i >= characters() )
+    internal_error( "delete_character, index out of bounds" );
   data.erase( data.begin() + i );
   }
 
 
-void Textline::swap_characters( const int i, const int j )
-						throw( Internal_error )
+void Textline::swap_characters( int i, int j ) throw()
   {
-  if( i > characters() || j > characters() )
-    throw Internal_error( "character index too big (swap_characters)" );
+  if( i < 0 || i >= characters() || j < 0 || j >= characters() )
+    internal_error( "swap_characters, index out of bounds" );
   Character tmp = data[i]; data[i] = data[j]; data[j] = tmp;
   }
 
 
-Character & Textline::character( const int i ) const throw( Internal_error )
+Character & Textline::character( int i ) const throw()
   {
-  if( i >= characters() ) throw Internal_error( "character index too big" );
+  if( i < 0 || i >= characters() )
+    internal_error( "character, index out of bounds" );
   return data[i];
   }
 
@@ -127,8 +128,7 @@ void Textline::join( Textline & l ) throw()
   }
 
 
-void Textline::print( FILE * outfile, const bool graph, const bool recursive )
-								const throw()
+void Textline::print( FILE * outfile, bool graph, bool recursive ) const throw()
   {
   if( graph || recursive ) fprintf( outfile, "mean_height = %d\n", mean_height() );
 
@@ -161,10 +161,11 @@ void Textline::recognize1() const throw()
   int dx = xr - xl, dy = yr - yl;
   for( int i = 0; i < characters(); ++i )
     {
+    Character & c = character( i );
     int charbox_vcenter = yl;
     if( dx )
-      charbox_vcenter += ( dy * ( character( i ).hcenter() - xl ) / dx );
-    character( i ).recognize1( charbox_vcenter );
+      charbox_vcenter += ( dy * ( c.hcenter() - xl ) / dx );
+    c.recognize1( charbox_vcenter );
     }
   }
 
@@ -173,28 +174,73 @@ void Textline::recognize2() throw()
   {
   if( characters() == 0 ) return;
 
+  // separate merged characters recognized by recognize1
+  for( int i = 0; i < characters(); )
+    {
+    const Character & c = character( i );
+    if( c.guesses() > 2 && c.guess( 0 ).ch == 0 && c.blocks() >= 1 )
+      {
+      int g, left = c.guess( 0 ).value;
+      for( g = 1; g < c.guesses(); ++g )
+        {
+        const Character & c = character( i );
+        const Block & b = c.block_list().front();
+        Rectangle r( left, b.top(), c.guess( g ).value, b.bottom() );
+        Character c1( Block( r, *c.blockmap(), b.id() ) );
+        std::list< Block >::const_iterator p = c.block_list().begin();
+        for( ++p; p != c.block_list().end(); ++p )
+          if( r.includes_hcenter( *p ) ) c1.add_block( *p );
+        c1.add_guess( c.guess( g ).ch, 0 );
+        left = c.guess( g ).value + 1;
+        insert_character( i + g, c1 );
+        }
+      delete_character( i ); i += g - 1;
+      }
+    else ++i;
+    }
+
+  // try to recognize separately the 2 overlapped blocks of an
+  // unrecognized character
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( c.guesses() == 0 && c.block_list().size() == 2 &&
+        c.block_list().front().v_overlaps( c.block_list().back() ) )
+      {
+      Character c1( c.block_list().front() );
+      Character c2( c.block_list().back() );
+      c1.recognize1( c.vcenter() );
+      c2.recognize1( c.vcenter() );
+      if( c1.left() <= c2.left() ) { c = c1; insert_character( i + 1, c2 ); }
+      else { c = c2; insert_character( i + 1, c1 ); }
+      ++i;
+      }
+    }
+
   // transform some small letters to capitals
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
-    if( c1.guess_map().size() == 1 )
+    if( c1.guesses() == 1 )
       {
-      char ch = c1.guess_map().begin()->first;
+      unsigned char ch = c1.guess( 0 ).ch;
       if( isspace( ch ) ) { begin = i + 1 ; continue; }
-      if( ch != 'c' && ch != 'o' && ch != 's' && ch != 'v' && ch != 'w' &&
-          ch != 'x' && ch != 'z' ) continue;
+      if( ch != 'c' && ch != 'o' && ch != 's' && ch != 'u' && ch != 'v' &&
+          ch != 'w' && ch != 'x' && ch != 'z' ) continue;
       if( 4 * c1.height() > 5 * mean_height() )
         { c1.only_guess( toupper( ch ), 0 ); continue; }
       if( 5 * c1.height() < 4 * mean_height() ) continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
-        if( c2.guess_map().size() >= 1 )
+        if( c2.guesses() >= 1 )
           {
-          char ch2 = c2.guess_map().begin()->first;
+          unsigned char ch2 = c2.guess( 0 ).ch;
           if( isspace( ch2 ) ) break;
-          if( isupper( ch2 ) && similar( c1.height(), c2.height(), 10 ) )
-            { c1.add_guess( toupper( ch ), 1 ); break; }
+          if( ( isalpha( ch2 ) && 4 * c1.height() > 5 * c2.height() ) ||
+              ( isupper( ch2 ) && ch2 != 'B' &&
+                similar( c1.height(), c2.height(), 10 ) ) )
+            { c1.insert_guess( 0, toupper( ch ), 1 ); break; }
           }
         }
       }
@@ -204,11 +250,11 @@ void Textline::recognize2() throw()
   for( int i = characters() - 1; i > 0; --i )
     {
     Character & c1 = character( i - 1 );
-    if( c1.guess_map().size() == 1 && c1.guess_map().begin()->first == 'p' )
+    if( c1.guesses() == 1 && c1.guess( 0 ).ch == 'p' )
       {
       Character & c2 = character( i );
-      if( c2.guess_map().size() == 0 ) continue;
-      switch( c2.guess_map().begin()->first )
+      if( c2.guesses() == 0 || !isalnum( c2.guess( 0 ).ch ) ) continue;
+      switch( c2.guess( 0 ).ch )
         {
         case 'g': case 'j': case 'p': case 'q': case 'y':
                   if( c1.bottom() + 2 > c2.bottom() ) continue; break;
@@ -219,23 +265,34 @@ void Textline::recognize2() throw()
       }
     }
 
-  // transform small o with accent or diaeresis to capital
-  for( int i = 0; i < characters(); ++i )
+  // transform small o or u with accent or diaeresis to capital
+  for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
-    if( c1.block_list().size() >= 2 && c1.guess_map().size() == 1 )
+    if( c1.guesses() >= 1 )
       {
-      char ch = c1.guess_map().begin()->first;
-      if( ch != '' && ch != '' && ch != '' && ch != '' ) continue;
-      const Block & b = c1.block_list().front();
-      if( 4 * b.height() > 5 * mean_height() )
-        switch( ch )
+      unsigned char ch = c1.guess( 0 ).ch;
+      if( isspace( ch ) ) { begin = i + 1 ; continue; }
+      if( ch <= 127 || c1.block_list().size() < 2 ) continue;
+      int chb = ISO_8859_1::base_letter( ch );
+      if( chb != 'o' && chb != 'u' ) continue;
+      const Block & b1 = c1.block_list().front();
+      for( int j = begin; j < characters(); ++j ) if( j != i )
+        {
+        Character & c2 = character( j );
+        if( c2.guesses() >= 1 )
           {
-          case '': c1.only_guess( '', 0 ); break;
-          case '': c1.only_guess( '', 0 ); break;
-          case '': c1.only_guess( '', 0 ); break;
-          case '': c1.only_guess( '', 0 ); break;
+          unsigned char ch2 = c2.guess( 0 ).ch;
+          int ch2b = ISO_8859_1::base_letter( ch2 );
+          if( !ch2b && ch2 > 127 ) continue;
+          if( isspace( ch2 ) ) break;
+          if( ( isalpha( ch2 ) && 4 * b1.height() > 5 * c2.height() ) ||
+              ( isupper( ch2 ) && similar( b1.height(), c2.height(), 10 ) ) ||
+              ( isalpha( ch2b ) && 4 * c1.height() > 5 * c2.height() ) ||
+              ( isupper( ch2b ) && similar( c1.height(), c2.height(), 10 ) ) )
+            { c1.insert_guess( 0, ISO_8859_1::toupper( ch ), 1 ); break; }
           }
+        }
       }
     }
 
@@ -244,34 +301,49 @@ void Textline::recognize2() throw()
     {
     Character & c1 = character( i );
     Character & c2 = character( i + 1 );
-    if( c1.guess_map().size() == 1 && c2.guess_map().size() == 1 )
+    if( c1.guesses() == 1 && c2.guesses() == 1 )
       {
-      char ch1 = c1.guess_map().begin()->first;
-      char ch2 = c2.guess_map().begin()->first;
+      unsigned char ch1 = c1.guess( 0 ).ch;
+      unsigned char ch2 = c2.guess( 0 ).ch;
       if( ( ch1 == '\'' || ch1 == '`' ) && ch1 == ch2 &&
-          similar( c1.width(), c2.left() - c1.right(), 60 ) )
+          2 * ( c2.left() - c1.right() ) < 3 * c1.width() )
         { c1.join( c2 ); c1.only_guess( '"', 0 ); delete_character( i + 1 ); }
       }
     }
 
+  // join a comma followed by a period into a semicolon
+  for( int i = 0; i < characters() - 1; ++i )
+    {
+    Character & c1 = character( i );
+    Character & c2 = character( i + 1 );
+    if( c1.guesses() == 1 && c2.guesses() == 1 )
+      {
+      unsigned char ch1 = c1.guess( 0 ).ch;
+      unsigned char ch2 = c2.guess( 0 ).ch;
+      if( ch1 == ',' && ch2 == '.' && c1.top() > c2.bottom() &&
+          c2.left() - c1.right() < c2.width() )
+        { c1.join( c2 ); c1.only_guess( ';', 0 ); delete_character( i + 1 ); }
+      }
+    }
+
   // transform 'O' or 'l' into '0' or '1'
   for( int i = 0, begin = 0; i < characters(); ++i )
     {
     Character & c1 = character( i );
-    if( c1.guess_map().size() == 1 )
+    if( c1.guesses() == 1 )
       {
-      char ch = c1.guess_map().begin()->first;
+      unsigned char ch = c1.guess( 0 ).ch;
       if( isspace( ch ) ) { begin = i + 1 ; continue; }
       if( ch != 'O' && ch != 'l' ) continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
-        if( c2.guess_map().size() >= 1 )
+        if( c2.guesses() >= 1 )
           {
-          char ch2 = c2.guess_map().begin()->first;
+          unsigned char ch2 = c2.guess( 0 ).ch;
           if( isspace( ch2 ) ) break;
           if( isdigit( ch2 ) && similar( c1.height(), c2.height(), 10 ) )
-            { c1.add_guess( (ch == 'l') ? '1' : '0', 1 ); break; }
+            { c1.insert_guess( 0, (ch == 'l') ? '1' : '0', 1 ); break; }
           }
         }
       }
